// Created by Kaden Werner and Alex Morin, April 2022.
// See documentation on Instructables.com for code references

#include <PID_v1.h> 
#include "Wire.h" // for communication with IMU
#include <MPU6050_light.h> // for scaling raw IMU data to degrees
#include <PWMServo.h> // for servo control

//General setup______________________________________________________________________
int motor=11;// ESC data line, pin D11
int motor_speed = 114; // minimum PMW value to turn on motor (lowest speed)

PWMServo servo_cradle; // declare servo object to be used by PWMServo


//PID _______________________________________________________________________________
double SetP, PV, CV; // declare setpoint, process variable and control variable objects
double Pk1 = 0.6667;  //gain
double Ik1 = 0.03; //integral
double Dk1 = 0.0; // derivative ( off )

PID PID1(&PV, &CV, &SetP, Pk1, Ik1 , Dk1, DIRECT); // attach variables to PID1 func.

//IMU data___________________________________________________________________________

MPU6050 mpu(Wire); // for communication with IMU
unsigned long timer = 0; //used for timing updates

void setup() //executes once upon energizing 
{
   
  pinMode(motor, OUTPUT);
  analogWrite(motor, 113); //idle value for motors ESC
  delay(5000); // gives ESC enough time to recognize idle command as it starts up

  servo_cradle.attach(10);
  servo_cradle.write(85); //closest center value achievable with barbs on servo arm
  
//PID________________________________________________________________________________
  PID1.SetMode(AUTOMATIC); // ALEX probably change this to MANUAL and have the HMI change to AUTOMATIC              
  PID1.SetOutputLimits(-25, 25);
  PID1.SetSampleTime(10); // milliseconds

//IMU________________________________________________________________________________
  Wire.begin();
  byte status = mpu.begin(); 
  while(status!=0){ } // puts controller into infinite nothing loop until it can connect to IMU
  mpu.calcOffsets(); // physical position at this time will generate zero
  
}
void loop() //continuous control loop
{
  //IMU______________________________________________________________________________
  mpu.update();  
  if((millis()-timer)>10)// get data every 10ms
  { 
  PV=(mpu.getAngleY()); // update PV
  timer = millis();  
  }
  
  //PID control______________________________________________________________________
  SetP = 0.0; //declare setpoint to be 0 degrees (centered)
  if(PV>2 or PV<-2) // creates +/- 2 degree deadband about center
  {
  PID1.Compute(); // PID updates inputs and output values
  }

  //Servo____________________________________________________________________________
  int servo_pos = map(CV,-25,25,60,110); //scale PID output to +/- 25 degrees about centerpoint 
  servo_cradle.write(servo_pos); // update servo position

  
  //Motor control____________________________________________________________________
  analogWrite(motor, motor_speed); //motor just turns on at lowest RPM setting
}
